/* Copyright (C) 2012 Carlos Pais 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef UTILS_H
#define UTILS_H

#include <QColor>
#include <QVariant>
#include <QFile>
#include <QDir>
#include <QDebug>

namespace Utils
{

inline QColor listToColor(const QVariantList& color)
{
    if (color.size() != 4)
        return QColor();

    bool ok = true;

    int red = color.at(0).toInt(&ok);
    if (! ok)
        return QColor();

    int green = color.at(1).toInt(&ok);
    if (! ok)
        return QColor();

    int blue = color.at(2).toInt(&ok);
    if (! ok)
        return QColor();

    int alpha = color.at(3).toInt(&ok);
    if (! ok)
        return QColor(red, green, blue, 255);

    return QColor(red, green, blue, alpha);
}

inline QVariantList colorToList(const QColor& color)
{
    QVariantList _color;
    _color << color.red() << color.green() << color.blue()
             << color.alpha();
    return _color;
}

inline QString colorToString(const QColor& color)
{
    return QString("(%1, %2, %3, %4)").arg(color.red())
                                        .arg(color.green())
                                        .arg(color.blue())
                                        .arg(color.alpha());
}

inline int lastDigitPosition(const QString& text)
{
    int j;

    for(j=text.size()-1; j >= 0; --j)
        if (text[j].isDigit())
            break;

    return j;
}

inline QString incrementLastDigit(QString text)
{
    int j = lastDigitPosition(text);

    if (j == -1)
        text += "1";
    else {
        int digit = text[j].digitValue();
        digit++;
        text.replace(j, 1, QString::number(digit));
    }

    return text;
}

inline void safeCopy(const QString& src, const QString& dest)
{
    if (! QFile::exists(src))
        return;

    if (QFile::exists(dest)) {
        if (QFile::exists(dest+".bkp"))
            QFile(dest + ".bkp").remove();

        QFile::copy(dest, dest + ".bkp");
        QFile(dest).remove();
    }

    QFile::copy(src, dest);
}

inline bool isNumber(const QString& str)
{
    for (int i=0; i < str.size(); i++)
        if (! str[i].isDigit())
            return false;

    return true;
}

//Copied from: http://john.nachtimwald.com/2010/06/08/qt-remove-directory-and-its-contents/
inline bool removeDir(const QString &dirName)
{
    bool result = true;
    QDir dir(dirName);

    if (dir.exists(dirName)) {
        Q_FOREACH(QFileInfo info, dir.entryInfoList(QDir::NoDotAndDotDot | QDir::System | QDir::Hidden  | QDir::AllDirs | QDir::Files, QDir::DirsFirst)) {
            if (info.isDir()) {
                result = removeDir(info.absoluteFilePath());
            }
            else {
                result = QFile::remove(info.absoluteFilePath());
            }

            if (!result) {
                return result;
            }
        }
        result = dir.rmdir(dirName);
    }

    return result;
}

}

#endif // UTILS_H
